/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include "iwlib.h"  /* Header */

/*
 * Wireless Tools
 *
 *  Jean II - HPLB 97->99 - HPL 99->01
 *
 * Main code for "iwconfig". This is the generic tool for most
 * manipulations...
 * You need to link this code against "iwlib.c" and "-lm".
 *
 * This file is released under the GPL license.
 *     Copyright (c) 1997-2002 Jean Tourrilhes <jt@hpl.hp.com>
 *
 * Get wireless informations & config from the device driver
 * We will call all the classical wireless ioctl on the driver through
 * the socket to know what is supported and to get the settings...
 */
static int
get_info(int   skfd,
  char *   ifname,
  struct wireless_info * info)
{
  struct iwreq  wrq;

  memset((char *) info, 0, sizeof(struct wireless_info));

  /* Get wireless name */
  if(iw_get_ext(skfd, ifname, SIOCGIWNAME, &wrq) < 0)
    {
      /* If no wireless name : no wireless extensions */
      /* But let's check if the interface exists at all */
      struct ifreq ifr;

      strcpy(ifr.ifr_name, ifname);
      if(ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0){

   perror("socket");
   return(-ENODEV);}
      else
 return(-ENOTSUP);
    }
  else
    {
      strncpy(info->name, wrq.u.name, IFNAMSIZ);
      info->name[IFNAMSIZ] = '\0';
    }

  /* Get ranges */
  if(iw_get_range_info(skfd, ifname, &(info->range)) >= 0)
    info->has_range = 1;

  /* Get network ID */
  if(iw_get_ext(skfd, ifname, SIOCGIWNWID, &wrq) >= 0)
    {
      info->has_nwid = 1;
      memcpy(&(info->nwid), &(wrq.u.nwid), sizeof(iwparam));
    }

  /* Get frequency / channel */
  if(iw_get_ext(skfd, ifname, SIOCGIWFREQ, &wrq) >= 0)
    {
      info->has_freq = 1;
      info->freq = iw_freq2float(&(wrq.u.freq));
    }

  /* Get sensitivity */
  if(iw_get_ext(skfd, ifname, SIOCGIWSENS, &wrq) >= 0)
    {
      info->has_sens = 1;
      memcpy(&(info->sens), &(wrq.u.sens), sizeof(iwparam));
    }

  /* Get encryption information */
  wrq.u.data.pointer = (caddr_t) info->key;
  wrq.u.data.length = IW_ENCODING_TOKEN_MAX;
  wrq.u.data.flags = 0;
  if(iw_get_ext(skfd, ifname, SIOCGIWENCODE, &wrq) >= 0)
    {
      info->has_key = 1;
      info->key_size = wrq.u.data.length;
      info->key_flags = wrq.u.data.flags;
    }

  /* Get ESSID */
  wrq.u.essid.pointer = (caddr_t) info->essid;
  wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
  wrq.u.essid.flags = 0;
  if(iw_get_ext(skfd, ifname, SIOCGIWESSID, &wrq) >= 0)
    {
      info->has_essid = 1;
      info->essid_on = wrq.u.data.flags;
    }

  /* Get AP address */
  if(iw_get_ext(skfd, ifname, SIOCGIWAP, &wrq) >= 0)
    {
      info->has_ap_addr = 1;
      memcpy(&(info->ap_addr), &(wrq.u.ap_addr), sizeof (sockaddr));
    }

  /* Get NickName */
  wrq.u.essid.pointer = (caddr_t) info->nickname;
  wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
  wrq.u.essid.flags = 0;
  if(iw_get_ext(skfd, ifname, SIOCGIWNICKN, &wrq) >= 0)
    if(wrq.u.data.length > 1)
      info->has_nickname = 1;

  /* Get bit rate */
  if(iw_get_ext(skfd, ifname, SIOCGIWRATE, &wrq) >= 0)
    {
      info->has_bitrate = 1;
      memcpy(&(info->bitrate), &(wrq.u.bitrate), sizeof(iwparam));
    }

  /* Get RTS threshold */
  if(iw_get_ext(skfd, ifname, SIOCGIWRTS, &wrq) >= 0)
    {
      info->has_rts = 1;
      memcpy(&(info->rts), &(wrq.u.rts), sizeof(iwparam));
    }

  /* Get fragmentation threshold */
  if(iw_get_ext(skfd, ifname, SIOCGIWFRAG, &wrq) >= 0)
    {
      info->has_frag = 1;
      memcpy(&(info->frag), &(wrq.u.frag), sizeof(iwparam));
    }

  /* Get operation mode */
  if(iw_get_ext(skfd, ifname, SIOCGIWMODE, &wrq) >= 0)
    {
      info->mode = wrq.u.mode;
      if((info->mode < IW_NUM_OPER_MODE) && (info->mode >= 0))
 info->has_mode = 1;
    }

  /* Get Power Management settings */
  wrq.u.power.flags = 0;
  if(iw_get_ext(skfd, ifname, SIOCGIWPOWER, &wrq) >= 0)
    {
      info->has_power = 1;
      memcpy(&(info->power), &(wrq.u.power), sizeof(iwparam));
    }

#if WIRELESS_EXT > 9
  /* Get Transmit Power */
  if(iw_get_ext(skfd, ifname, SIOCGIWTXPOW, &wrq) >= 0)
    {
      info->has_txpower = 1;
      memcpy(&(info->txpower), &(wrq.u.txpower), sizeof(iwparam));
    }
#endif

#if WIRELESS_EXT > 10
  /* Get retry limit/lifetime */
  if(iw_get_ext(skfd, ifname, SIOCGIWRETRY, &wrq) >= 0)
    {
      info->has_retry = 1;
      memcpy(&(info->retry), &(wrq.u.retry), sizeof(iwparam));
    }
#endif /* WIRELESS_EXT > 10 */

  /* Get stats */
  if(iw_get_stats(skfd, ifname, &(info->stats)) >= 0)
    {
      info->has_stats = 1;
    }

  return(0);
}

/*------------------------------------------------------------------*/
/*
 * Macro to handle errors when setting WE
 * Print a nice error message and exit...
 * We define them as macro so that "return" do the right thing.
 * The "do {...} while(0)" is a standard trick
 */
#define ERR_SET_EXT(rname, request) \
 fprintf(stderr, "Error for wireless request \"%s\" (%X) :\n", \
  rname, request)

#define ABORT_ARG_NUM(rname, request) \
 do { \
  ERR_SET_EXT(rname, request); \
  fprintf(stderr, "    too few arguments.\n"); \
  return(-1); \
 } while(0)

#define ABORT_ARG_TYPE(rname, request, arg) \
 do { \
  ERR_SET_EXT(rname, request); \
  fprintf(stderr, "    invalid argument \"%s\".\n", arg); \
  return(-2); \
 } while(0)

#define ABORT_ARG_SIZE(rname, request, max) \
 do { \
  ERR_SET_EXT(rname, request); \
  fprintf(stderr, "    argument too big (max %d)\n", max); \
  return(-3); \
 } while(0)

/*------------------------------------------------------------------*/
/*
 * Wrapper to push some Wireless Parameter in the driver
 * Use standard wrapper and add pretty error message if fail...
 */
#define IW_SET_EXT_ERR(skfd, ifname, request, wrq, rname) \
 do { \
 if(iw_set_ext(skfd, ifname, request, wrq) < 0) { \
  ERR_SET_EXT(rname, request); \
  fprintf(stderr, "    SET failed on device %-1.16s ; %s.\n", \
   ifname, strerror(errno)); \
  return(-5); \
 } } while(0)

/*------------------------------------------------------------------*/
/*
 * Wrapper to extract some Wireless Parameter out of the driver
 * Use standard wrapper and add pretty error message if fail...
 */
#define IW_GET_EXT_ERR(skfd, ifname, request, wrq, rname) \
 do { \
 if(iw_get_ext(skfd, ifname, request, wrq) < 0) { \
  ERR_SET_EXT(rname, request); \
  fprintf(stderr, "    GET failed on device %-1.16s ; %s.\n", \
   ifname, strerror(errno)); \
  return(-6); \
 } } while(0)


/******************************* MAIN ********************************/

/*------------------------------------------------------------------*/
/*
 * The main !
 */
void iw::process_signal()
{
     if ( flag ) {
         timerid = startTimer( 1000 );
         startButton->setText("Stop");
         flag = 0;
     } else {
         killTimer( timerid );
         startButton->setText("Start");
         flag = 1;  
     }    
}

void iw::init()
{
     flag = 1;
}


void iw::timerEvent( QTimerEvent * )
{
      int skfd;             
      if((skfd = iw_sockets_open()) < 0)
      {
        perror("socket");
        exit(-1);
      }
      
      char    buff[1024];
      struct ifconf ifc;
      struct ifreq *ifr;
      int i; 

      ifc.ifc_len = sizeof(buff);
      ifc.ifc_buf = buff;
      if(ioctl(skfd, SIOCGIFCONF, &ifc) < 0)
      {
        fprintf(stderr, "SIOCGIFCONF: %s\n", strerror(errno));
        exit(-1);
      }
      ifr = ifc.ifc_req+2;
      
      int cont = 0;
      //for(i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++)
          if ( ! strcmp(ifr->ifr_name, ifaceEdit->text()) )
                cont=1;
      
      if (! cont) {    fprintf(stderr, "Interface not found\n"); exit(0); }

      char iface[100];
      struct wireless_info * info=new wireless_info;
      sprintf(iface, "%s", ifr->ifr_name);

      i=get_info(skfd, iface, info);
      essidEdit->setText(info->essid);
      modeEdit->setText(iw_operation_mode[info->mode]);
      chanEdit->setText( QString::number(int(info->freq), 10) );
      rtsEdit->setText( QString::number(int(info->rts.value), 10) );
   
      iwqual * qual = &info->stats.qual;
      iwrange * range = &info->range; 
      if(qual->level > range->max_qual.level) {
         qualdial->setValue(qual->qual);
         signaldial->setValue(qual->level - 256);
         textLabel1->setText( QString::number(qual->qual, 10)+"%" );
         textLabel2_6->setText( QString::number(qual->level - 256, 10)+" dBm" );
      }
   
      noiseEdit->setText( QString::number( qual->noise - 0x100, 10) );
      nwidEdit->setText( QString::number( info->stats.discard.nwid, 10) );
      beacEdit->setText( QString::number( info->stats.miss.beacon, 10) );
      
//  close(skfd);
}
